// ---------------------------------------------------------------------------
// fast_faves.cs -- Version 7.6 -- January 2, 2001
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// Buy stuff from inventory stations faster (select what you want in advance)
//
// IMPORTANT: BACKUP YOUR FAVORITES?.CS FILES BEFORE USING THIS SCRIPT
//            IF YOU ARE UPGRADING FROM FAST_FAVORITES.CS
//
//    --->    fast_faves.cs automatically converts your old favorites files
//            to the new format for you, but the conversion will only work
//            if you connect to a server that has all the items listed in
//            all of your loadouts (in all of your favorites files).  The
//            conversion WILL NOT WORK if you have favorites files with
//            items from more than 1 mod.
//
//            All old-format favorites?.cs files in your config directory
//            are converted when you connect to a server or host a game.
//
//   Example: if you connect to a server with only Base items, your
//            renegades loadouts in favorites3.cs (or whatever) won't be
//            converted properly.  Similarly, if you connect to a server
//            running Shifter (or some other mod), your Base loadouts in
//            favorites1.cs (or whatever) won't be converted properly.
//
//            This can really mess up your favorites, and the only way
//            out is to rebuild your favorites in the new format (either
//            in-game or by hand) -- so MAKE BACKUPS! :)
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\fast_faves.cs", "7.6", "Lorne Laliberte", "Jan 2, 2001");

include("writer\\event.cs");
include("writer\\inventory.cs");
include("writer\\item_events.cs");
include("writer\\loadout.cs");
include("writer\\menu.cs");
include("writer\\modifier.cs");
include("writer\\ping.cs");
include("writer\\queue.cs");
include("writer\\save.cs");
include("writer\\schedule.cs");
include("writer\\station_events.cs");
include("writer\\whatif.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_zoom.cs file

    // Set this to true if you want to force shutting the cursor off on eventExitStation.
    // This might solve the "exit station, get cursor on screen" bug which should not
    // happen unless you've enabled station_types.cs or have some other script that is
    // messing with things.  (I have been unable to duplicate the bug with the new fast
    // faves, under any circumstances, with only my scripts installed.)
    $FastFaves::ForceCursorOff = "false";

    // Set this to true to automatically buy the currently selected loadout
    // as soon as you access an inventory station.
    $FastFaves::AutoBuy = "true";

    // Set this to true to automatically buy the currently selected vehicle
    // whenever you access a vehicle station.
    $FastFaves::AutoBuyVehicle = "true";

    // Set this to true to automatically announce that an APC is ready to go
    // whenever you buy a vehicle
    $FastFaves::AnnounceAPC = "true";

    // Set this to true to prevent the inventory menu from popping up automatically
    // when you access an inventory station.  Note that it will still pop up momentarily
    // at vehicle stations, and at inventory stations on some servers. In those cases,
    // fast_favorites automatically closes the screen for you.  Note also that you can
    // always access the inventory menu using the "I" key.
    //$pref::noEnterInvStation = "true";

    // Set this to true to automatically close the inv menu as soon as it's displayed
    // at vehicle stations (also useful on servers that don't respect $pref::noEnterInvStation)
    $FastFaves::closeStation = "true";

    // Set favorites file to use when you first load Tribes
    // Setting this to 0 will load config\favorites0.cs
    // Setting this to 1 will load config\favorites1.cs, etc.
    $FastFaves::FileNumber = "0";

    // Set this to true to not reset your current loadout to the file's default
    // when you change to a different favorites?.cs file
    $FastFaves::RememberCurrentLoadoutBetweenFavoritesFiles = "false";

    // Note: the actual favorites key combos are defined in the favorites?.cs files

    // Toggle the inventory screen on/off at any time
    //bind(keyboard0, make, "i", to, "$Mode::ExitInventory=false; remoteEval(2048, ToggleInventoryMode);");

    // Toggle auto-buying mode on/off (automatically buy selected loadout)
    //bind(keyboard0, make, alt, "numpad*", TO, "FastFaves::AutoBuyToggle();");
    //bind(keyboard0, break, alt, "numpad*", TO, "", FastFaves::AutoBuyToggle_break);

    // Press this key to select a favorites?.cs file (using a menu, no less :)
    //bind(keyboard0, make, alt, "numpad+", TO, "FastFaves::selectFavoritesFile();");
    //bind(keyboard0, break, alt, "numpad+", TO, "", FastFaves::selectFavoritesFile_break);

    // Enter FastFaves config mode (edit/create loadouts from in game)
    //bind(keyboard0, make, control, "numpadenter", TO, "FastFaves::ConfigMode();");
    //bind(keyboard0, break, control, "numpadenter", TO, "", FastFaves::ConfigMode_break);

    // FastFaves HUD settings
    // You probably won't want to change these, but feel free if you know what you're doing :)

    $FastFavesMAIN::x = "50%";
    $FastFavesMAIN::y = "80%";
    $FastFavesMAIN::width = "50%";
   
    $FastFavesEXTRAS::x = "l:FastFavesMAIN";
    $FastFavesEXTRAS::y = "s:FastFavesMAIN";
    $FastFavesEXTRAS::width = "20%";

    $FastFavesWEAPONS::x = "r:FastFavesMAIN";
    $FastFavesWEAPONS::y = "s:FastFavesMAIN";
    $FastFavesWEAPONS::width = "20%";

// ---------------------------------------------------------------------------

WriterPrefs::load("fast_faves");

// ---------------------------------------------------------------------------
// HUD settings
// ---------------------------------------------------------------------------

$FastFavesMAIN::height = 7 * 14;

$FastFavesEXTRAS::height = 7 * 14;
$FastFavesWEAPONS::height = 7 * 14;

// ---------------------------------------------------------------------------
// SETUP
// ---------------------------------------------------------------------------

$FastFaves::FileNumber += 0; // make sure it's numeric

function FastFaves::bind(%device, %m1, %m2, %m3, %key)
{
    if($FastFaves::exists[%device, %m1, %m2, %m3, %key])
        return;

    // add this key to the list of keys used

    $FastFaves::exists[%device, %m1, %m2, %m3, %key] = true;

    $FastFaves::list::count++;

    $FastFaves::list::device[$FastFaves::list::count] = %device;
    $FastFaves::list::m1[$FastFaves::list::count] = %m1;
    $FastFaves::list::m2[$FastFaves::list::count] = %m2;
    $FastFaves::list::m3[$FastFaves::list::count] = %m3;
    $FastFaves::list::key[$FastFaves::list::count] = %key;

    // bind the key
    bind(%device, make, %m1, %m2, %m3, %key, to, "FastFaves(" @ %device @ ", \"" @ %m1 @ "\", \""
                                                                                 @ %m2 @ "\", \""
                                                                                 @ %m3 @ "\", \""
                                                                                 @ %key @ "\");");
}


// Load the favorites?.cs file
function FastFaves::setupFavorites()
{
    echo();
    echo("Setting up favorites...");

    if(!isFile("config\\favorites0.cs"))
    {
        echo();
        echo(" --> creating default config\\favorites0.cs file <--");
        echo();

        // create default favorites0.cs
        $FastFaves::CurrentFileNumber = 0;
        exec("writer\\default_favorites0.cs");
        FastFaves::saveData();
    }

    if(!$FastFaves::SetupComplete)
    {
        echo("Setting up list of favorites files...");

        $FastFaves::CurrentFileNumber = 0;
        while(include("favorites" @ $FastFaves::CurrentFileNumber @ ".cs", force) != notfound)
        {
            if($FastFaves::oldFavoritesFile)
            {
                echo("  Converting " @ $FastFaves::Favorites[$FastFaves::CurrentFileNumber]);

                // we've converted an old favorites file, so let's save the converted data
                FastFaves::saveData();
                $FastFaves::oldFavoritesFile = false;
                deleteVariables("$FastFaves::loadout*");

                echo("  Adding " @ $FastFaves::Favorites[$FastFaves::CurrentFileNumber] @ " to favorites list");
            }
            else
            {
                echo("  Adding " @ $FastFaves::Favorites[$FastFaves::CurrentFileNumber] @ " to favorites list");
            }
            $FastFaves::CurrentFileNumber++;
        }
        $FastFaves::MaxFileNumber = $FastFaves::CurrentFileNumber - 1;

        // remove any old favorites data
        deleteVariables("$FastFaves::key*");
    }

    $FastFaves::SetupComplete = true;

    $FastFaves::CurrentFileNumber = $FastFaves::FileNumber;

    // now let's load the desired favorites file
    include("favorites" @ $FastFaves::FileNumber @ ".cs", force);

    echo("Using config\\favorites" @ $FastFaves::FileNumber @ ".cs -- " @ $FastFaves::Favorites[$FastFaves::FileNumber]);
    echo();

    //isFile("config\\WriterPrefs.cs");

    if($FastFaves::SetupComplete)
        Event::Trigger(eventFavoritesInitialized);
}

function FastFaves::selectFavoritesFile()
{
    menu::reset();
    menu::copyArray("$FastFaves::Favorites[%1]", 0, $FastFaves::MaxFileNumber);
    menu::make("<jc><f1>Select favorites file or press <f2>Esc<f1> to exit\n", FastFaves::favoritesFile::onSelected, "", "", $FastFaves::CurrentFileNumber);
    return;
}

function FastFaves::favoritesFile::onSelected(%index, %str)
{
echo(%str);

    $FastFaves::CurrentFileNumber = %index;

    // load the desired favorites file
    include("favorites" @ %index @ ".cs", force);

    echo("Using config\\favorites" @ %index @ ".cs -- " @ $FastFaves::Favorites[%index]);
    echo();

    Event::Trigger(eventFavoritesInitialized);
    return;
}

function NameForTheseFavorites(%name)
{
    $FastFaves::oldFavoritesFile = false;

    if($FastFaves::SetupComplete)
        return;

    $FastFaves::Favorites[$FastFaves::CurrentFileNumber] = %name;
    return;
}

function FastFaves::saveData()
{
    flushExportText();
    newObject(fastfavesSave, fearguiformattedtext,0,0,0,0);


    AddExportText("// ----------------------------------------------------------------------------");
    AddExportText("// favorites0.cs -- Version 3.1 -- May 9, 2000");
    AddExportText("// by Lorne Laliberte (writer@planetstarsiege.com)");
    AddExportText("//");
    AddExportText("// http://www.writerscripts.com");
    AddExportText("// ----------------------------------------------------------------------------");
    AddExportText("");
    AddExportText("// Usage:");
    AddExportText("//");
    AddExportText("// $FastFaves::loadout[<device>, <modifier1>, <modifier2>, <modifier3>, <key>, <field>] = \"<value>\";");
    AddExportText("//");
    AddExportText("// <device>    - the device the key combination belongs to (keyboard0, mouse0, etc.)");
    AddExportText("// <modifier1> - a modifier like \"shift\" or \"l_alt\" -- use \"\" if key is unmodified");
    AddExportText("// <modifier2> - a modifier like \"shift\" or \"l_alt\" -- use \"\" if key is unmodified");
    AddExportText("// <modifier3> - a modifier like \"shift\" or \"l_alt\" -- use \"\" if key is unmodified");
    AddExportText("// <key>       - the Tribes name for a key, like \"h\" or \"numpad1\"");
    AddExportText("// <field>     - the loadout field (see below)");
    AddExportText("// <value>     - (see below)");
    AddExportText("//");
    AddExportText("// Fields:       Values:");
    AddExportText("//");
    AddExportText("// name        - an optional name for the loadout");
    AddExportText("// armor       - name of the armor to buy");
    AddExportText("// pack        - name of the pack to buy");
    AddExportText("// weapon1     - name of weapon to buy");
    AddExportText("// weapon2     - name of weapon to buy");
    AddExportText("// weapon3     - name of weapon to buy");
    AddExportText("// weapon4     - name of weapon to buy");
    AddExportText("// weapon5     - name of weapon to buy");
    AddExportText("// weapon6     - name of weapon to buy");
    AddExportText("// mount       - name of weapon to mount on exiting station");
    AddExportText("// grenades    - true/false/\"\" -- buy grenades with this loadout");
    AddExportText("// mines       - true/false/\"\" -- buy mines with this loadout");
    AddExportText("// beacons     - true/false/\"\" -- buy beacons with this loadout");
    AddExportText("// tlaser      - true/false/\"\" -- buy a targetting laser with this loadout");
    AddExportText("// extra1      - name of misc other item");
    AddExportText("// extra2      - name of misc other item");
    AddExportText("// extra3      - name of misc other item");
    AddExportText("// extra4      - name of misc other item");
    AddExportText("// extra5      - name of misc other item");
    AddExportText("// extra6      - name of misc other item");
    AddExportText("// extra7      - name of misc other item");
    AddExportText("// autodrop    - name of litterbug item group to \"always drop\" (when autodrop key is held down or toggle is active)");
    AddExportText("// litterbug   - name of litterbug item group to drop manually");
    AddExportText("// litternext  - true/false/\"\" -- automatically drop the litterbug item group on next station access");
    AddExportText("// vehicle     - name of vehicle to buy at vehicle stations");
    AddExportText("//");
    AddExportText("//");
    AddExportText("// Notes:");
    AddExportText("//");
    AddExportText("// \"Special\" (any key) modifiers are allowed for <modifier1> (according to");
    AddExportText("// the rules for bind() function binds).");
    AddExportText("//");
    AddExportText("// Modifier and key names that contain spaces (are there any?) or are escape");
    AddExportText("// characters need to quoted -- for example, the \"\\\\\" key. ");
    AddExportText("//");
    AddExportText("// You can keep any of your current loadout items by leaving that field blank");
    AddExportText("// (\"\").  For instance, if you want to change your pack and your weapons, but");
    AddExportText("// leave your armor and vehicle selections the same, leave the armor and");
    AddExportText("// vehicle fields blank.");
    AddExportText("//");
    AddExportText("// fast_faves.cs will automatically use buyFavorites instead of buyItem if");
    AddExportText("// if more than one type of item is given.  Using buyFavorites means any items");
    AddExportText("// in your inventory that aren't in the currently defined loadout will be");
    AddExportText("// removed.");
    AddExportText("//");
    AddExportText("// To buy a single item, leave all other item fields blank.");
    AddExportText("//");
    AddExportText("// Important:");
    AddExportText("//");
    AddExportText("// Each loadout definition must be followed by a FastFaves::Bind() call");
    AddExportText("// to bind the key for that loadout.  The usage is:");
    AddExportText("//");
    AddExportText("// $FastFaves::bind(<device>, <modifier1>, <modifier2>, <modifier3>, <key>);");
    AddExportText("");
    AddExportText("");
    AddExportText("//-----------------------------------------------------------------------------");
    AddExportText("// Settings:");
    AddExportText("//-----------------------------------------------------------------------------");
    AddExportText("");
    AddExportText("");
    AddExportText("// Name for this favorites file (this is the name that will appear in the");
    AddExportText("// favorites file menu where you can switch between files)");
    AddExportText("NameForTheseFavorites(\"" @ escapestring($FastFaves::Favorites[$FastFaves::CurrentFileNumber]) @ "\");");
    AddExportText("");
    AddExportText("/////////////////////////////////");
    AddExportText("if($FastFaves::SetupComplete){ // Don't edit this line :)");
    AddExportText("/////////////////////////////////");
    AddExportText("");
    AddExportText("// Which loadout to use by default (after connecting to a game)");

    ExportSetting("$FastFaves::default::device", "");
    ExportSetting("$FastFaves::default::modifier1", "");
    ExportSetting("$FastFaves::default::modifier2", "");
    ExportSetting("$FastFaves::default::modifier3", "");
    ExportSetting("$FastFaves::default::key", "");

    for(%i = 1; %i <= $FastFaves::list::count; %i++)
    {
        AddExportText("");
        AddExportText("//-----------------------------------------------------------------------------");
        AddExportText("");

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", name]       = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              escapestring($FastFaves::loadout[$FastFaves::list::device[%i],
                                                               $FastFaves::list::m1[%i],
                                                               $FastFaves::list::m2[%i],
                                                               $FastFaves::list::m3[%i],
                                                               $FastFaves::list::key[%i], name])));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", armor]      = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], armor]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", pack]       = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], pack]));

        for(%w = 1; %w <= 6; %w++)
        {
            AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", weapon" @ %w @ "]    = \"%6\";",
                                  $FastFaves::list::device[%i],
                                  $FastFaves::list::m1[%i],
                                  $FastFaves::list::m2[%i],
                                  $FastFaves::list::m3[%i],
                                  $FastFaves::list::key[%i],
                                  $FastFaves::loadout[$FastFaves::list::device[%i],
                                                      $FastFaves::list::m1[%i],
                                                      $FastFaves::list::m2[%i],
                                                      $FastFaves::list::m3[%i],
                                                      $FastFaves::list::key[%i], weapon @ %w]));
        }

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", mount]      = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], mount]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", grenades]   = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], grenades]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", mines]      = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], mines]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", beacons]    = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], beacons]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", tlaser]     = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], tlaser]));

        for(%e = 1; %e <= 7; %e++)
        {
            AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", extra" @ %e @ "]     = \"%6\";",
                                  $FastFaves::list::device[%i],
                                  $FastFaves::list::m1[%i],
                                  $FastFaves::list::m2[%i],
                                  $FastFaves::list::m3[%i],
                                  $FastFaves::list::key[%i],
                                  $FastFaves::loadout[$FastFaves::list::device[%i],
                                                      $FastFaves::list::m1[%i],
                                                      $FastFaves::list::m2[%i],
                                                      $FastFaves::list::m3[%i],
                                                      $FastFaves::list::key[%i], extra @ %e]));
        }

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", autodrop]   = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              escapestring($FastFaves::loadout[$FastFaves::list::device[%i],
                                                               $FastFaves::list::m1[%i],
                                                               $FastFaves::list::m2[%i],
                                                               $FastFaves::list::m3[%i],
                                                               $FastFaves::list::key[%i], autodrop])));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", litterbug]  = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              escapestring($FastFaves::loadout[$FastFaves::list::device[%i],
                                                               $FastFaves::list::m1[%i],
                                                               $FastFaves::list::m2[%i],
                                                               $FastFaves::list::m3[%i],
                                                               $FastFaves::list::key[%i], litterbug])));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", litternext] = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], litternext]));

        AddExportText(sprintf("$FastFaves::loadout[%1, \"%2\", \"%3\", \"%4\", \"%5\", vehicle]    = \"%6\";",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i],
                              $FastFaves::loadout[$FastFaves::list::device[%i],
                                                  $FastFaves::list::m1[%i],
                                                  $FastFaves::list::m2[%i],
                                                  $FastFaves::list::m3[%i],
                                                  $FastFaves::list::key[%i], vehicle]));

        AddExportText(sprintf("    FastFaves::bind(%1, \"%2\", \"%3\", \"%4\", \"%5\");",
                              $FastFaves::list::device[%i],
                              $FastFaves::list::m1[%i],
                              $FastFaves::list::m2[%i],
                              $FastFaves::list::m3[%i],
                              $FastFaves::list::key[%i]));
    }

    AddExportText("");
    AddExportText("//-----------------------------------------------------------------------------");
    AddExportText("");
    AddExportText("////");
    AddExportText("} // Don't edit this line :)");
    AddExportText("////");
    AddExportText("");

    AddExportText("// Remove the fastfavesSave object that gets created whenever this file is executed");
    AddExportText("if(isObject(fastfavesSave))");
    AddExportText("    deleteobject(fastfavesSave);");

    exportObjectToScript(fastfavesSave, "config\\favorites" @ $FastFaves::CurrentFileNumber @ ".cs", true);
    deleteobject(fastfavesSave);
    flushExportText();

    echo();
    echo("config\\favorites" @ $FastFaves::CurrentFileNumber @ ".cs saved!");
    echo();
}


// use this for the conversion function
function FastFaves::bindKey(%modifier, %key)
{
    // set flag to indicate this file needs to be saved
    $FastFaves::oldFavoritesFile = true;

    if(%key == "")
    {
        %key = %modifier;
        %modifier = "";
    }

    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, name]    = $FastFaves::key[%modifier, %key, loadout];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, armor]   = $Inv::Name[$FastFaves::key[%modifier, %key, armor]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, pack]    = $Inv::Name[$FastFaves::key[%modifier, %key, pack]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon1] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon1]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon2] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon2]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon3] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon3]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon4] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon4]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon5] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon5]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, weapon6] = $Inv::Name[$FastFaves::key[%modifier, %key, weapon6]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, mount]   = $Inv::Name[$FastFaves::key[%modifier, %key, mount]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, grenades]  = "";
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, mines]  = "";
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, beacons]  = "";
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, tlaser]  = "";
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra1]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra1]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra2]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra2]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra3]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra3]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra4]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra4]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra5]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra5]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra6]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra6]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, extra7]  = $Inv::Name[$FastFaves::key[%modifier, %key, extra7]];
    $FastFaves::loadout[keyboard0, %modifier, "", "", %key, vehicle] = $Inv::Name[$FastFaves::key[%modifier, %key, vehicle]];

    if(!String::ICompare(%key, $FastFaves::defaultKey)) // ==
    {
        if(!String::ICompare(%modifier, $FastFaves::defaultModifier)) // ==
        {
            $FastFaves::default::device = "keyboard0";
            $FastFaves::default::modifier1 = $FastFaves::defaultModifier;
            $FastFaves::default::modifier2 = "";
            $FastFaves::default::modifier3 = "";
            $FastFaves::default::key = $FastFaves::defaultKey;

            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, grenades] = $FastFaves::BuyGrenades;
            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, mines]    = $FastFaves::BuyMines;
            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, beacons]  = $FastFaves::BuyBeacons;
            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, tlaser]   = true;
            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, autodrop] = "2 Repair Packs";
            $FastFaves::loadout[keyboard0, %modifier, "", "", %key, vehicle]  = $Inv::Name[$FastFaves::Vehicle];
        }
    }

    FastFaves::bind(keyboard0, %modifier, "", "", %key);
}


function FastFaves::ShowFavoritesFile()
{
    remoteBP(2048, "<jc>Using config\\favorites" @ $FastFaves::FileNumber @ ".cs -- <f1>" @ $FastFaves::Favorites[$FastFaves::FileNumber], 4);
}
Event::Attach(eventFavoritesInitialized, "FastFaves::ShowFavoritesFile();");


function FastFaves::nextFavoritesFile()
{
    $FastFaves::FavoritesFile++;

    if($FastFaves::FavoritesFile > $FastFaves::MaxFavoritesFiles)
        $FastFaves::FavoritesFile = 0;

    FastFaves::setupFavorites();
}


function FastFaves::prevFavoritesFile()
{
    $FastFaves::FavoritesFile--;

    if($FastFaves::FavoritesFile < 0)
        $FastFaves::FavoritesFile = $FastFaves::MaxFavoritesFiles;

    FastFaves::setupFavorites();
}


// ---------------------------------------------------------------------------
// MAIN
// ---------------------------------------------------------------------------


// This gets called when you press one of the fast favorites key combos.
function FastFaves(%device, %m1, %m2, %m3, %key)
{
    if($FastFaves::Handling)
    {
        Queue::push(FastFaves_device, %device);
        Queue::push(FastFaves_m1, %m1);
        Queue::push(FastFaves_m2, %m2);
        Queue::push(FastFaves_m3, %m3);
        Queue::push(FastFaves_key, %key);
        return;
    }

    $FastFaves::doing::device = %device;
    $FastFaves::doing::m1 = %m1;
    $FastFaves::doing::m2 = %m2;
    $FastFaves::doing::m3 = %m3;
    $FastFaves::doing::key = %key;

    FastFaves::do(%device, %m1, %m2, %m3, %key);
    return;
}


// This gets called when you press one of the fast favorites key combos.
function FastFaves::do(%device, %m1, %m2, %m3, %key)
{
    // Keep track of how many items are defined in the loadout, so we can
    // determine whether to use buyFavorites or buyItem
    %items = 0;
    %single = "";

    %temp_name = $FastFaves::loadname;

    // note: can't use loadout_name because I need to be able to delete the $FastFaves::loadout[]
    // array without affecting $FastFaves::loadname
    $FastFaves::loadname = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, name];

    %noname = !String::Compare($FastFaves::loadname, ""); // ==

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, armor] != "")
    {
        %single =
        $FastFaves::armor = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, armor];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, pack] != "")
    {
        %single =
        $FastFaves::pack = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, pack];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon1] != "")
    {
        %weapons = 0;
        for(%i = 1; %i < 7; %i++)
        {
            $FastFaves::weapon[%i] = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon @ %i];

            if($FastFaves::weapon[%i] != "")
            {
                %single = $FastFaves::weapon[%i];
                %items++;
                %weapons++;
            }
        }

        // Sort the weapon names alphabetically
        sortArray("$FastFaves::weapon", 1, %weapons);

        // build a string to compare with $loadout::weaponString
        $FastFaves::weaponString = "";
        for(%i = 1; %i <= %weapons; %i++)
        {
            $FastFaves::weaponString = $FastFaves::weaponString @ $FastFaves::weapon[%i];
        }
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, mount] != "")
    {
        $FastFaves::mount = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, mount];
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, grenades] != "")
    {
        %single = "Grenade";
        $FastFaves::grenades = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, grenades];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, mines] != "")
    {
        %single = "Mine";
        $FastFaves::mines = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, mines];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, tlaser] != "")
    {
        %single = "Targeting Laser";
        $FastFaves::tlaser = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, tlaser];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, beacons] != "")
    {
        %single = "Beacon";
        $FastFaves::beacons = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, beacons];
        %items++;
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra1] != "")
    {
        %extras = 0;
        for(%i = 1; %i < 8; %i++)
        {
            if($FastFaves::NoUnwantedAmmo)
            {
                // reset (clear) the $FastFaves::WantedAmmo array
                if($Inv::isAmmo[$FastFaves::extra[%i]])
                    $FastFaves::WantedAmmo[$FastFaves::extra[%i]] = "";
            }

            $FastFaves::extra[%i] = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra @ %i];

            if($FastFaves::extra[%i] != "")
            {
                if($FastFaves::NoUnwantedAmmo)
                {
                    if($Inv::isAmmo[$FastFaves::extra[%i]])
                        $FastFaves::WantedAmmo[$FastFaves::extra[%i]] = true;
                }

                %single = $FastFaves::extra[%i];
                %items++;
                %extras++;
            }
        }

        // Sort the extra item names alphabetically
        sortArray("$FastFaves::extra", 1, %extras);

        // build a string to compare with $loadout::extraString
        $FastFaves::extraString = "";
        for(%i = 1; %i <= %extras; %i++)
        {
            $FastFaves::extraString = $FastFaves::extraString @ $FastFaves::extra[%i];
        }
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, autodrop] != "")
    {
        $FastFaves::autodrop = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, autodrop];
        $litterbug::autodrop = $litterbug[$FastFaves::autodrop];
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, litterbug] != "")
    {
        $FastFaves::litterbug = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, litterbug];
        $litterbug::current = $litterbug[$FastFaves::litterbug];
    }

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, litternext] != "")
    {
        $litterbug::next::enabled = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, litternext];
    }

    // restore the loadout name if we aren't changing any carried items
    if(!%items && %noname)
        $FastFaves::loadname = %temp_name;

    if($FastFaves::loadout[%device, %m1, %m2, %m3, %key, vehicle] != "")
    {
        $FastFaves::vehicle = $FastFaves::loadout[%device, %m1, %m2, %m3, %key, vehicle];
        if(!%items)
        {
            %single = $FastFaves::vehicle;
            %items = 1;
        }
    }

    FastFaves::validate();

    // Now, are we buying one item, or many?

    if(%items > 1) // many items
    {
        // buy the loadout
        if($Station::AtStation)
            FastFaves::buyFavorites();
        else
            FastFaves::PreloadMessage(5);
    }
    else if(%items) // one item
    {
        if($Station::AtStation)
        {
            if($FastFaves::AnnounceAPC)
            {
                if(!String::ICompare(%single, "LPC")) // ==
                {
                    if(Flood::protect(LPC_ready_message, 10))
                    {
                        if(String::ICompare($loadout::armor, "Light Armor")) // !=
                            schedule::add("say(1, \"LPC ready to go, waiting for 1 passenger + 1 pilot.~wwaitpas\");", 3, APC_ready_message);
                        else
                            schedule::add("say(1, \"LPC ready to go, waiting for 2 passengers.~wwaitpas\");", 3, APC_ready_message);
                    }
                }
                else if(!String::ICompare(%single, "HPC")) // ==
                {
                    if(Flood::protect(HPC_ready_message, 10))
                    {
                        if(String::ICompare($loadout::armor, "Light Armor")) // !=
                            schedule::add("say(1, \"HPC ready to go, waiting for 3 passengers + 1 pilot.~wwaitpas\");", 3, APC_ready_message);
                        else
                            schedule::add("say(1, \"HPC ready to go, waiting for 4 passengers.~wwaitpas\");", 3, APC_ready_message);
                    }
                }
            }

            $FastFaves::Handling++;
            buy(%single);

            // Make sure we get the max number of mines and grenades with an ammo pack
            if(!String::ICompare(%single, "Ammo Pack")) // ==
                schedule::add("buy(Beacon);", 0.1);
        }
        else
        {
            FastFaves::PreloadMessage(5);
        }
    }
    else // no items -- feed the station
    {
        $FastFaves::pack = "";
        $FastFaves::weapon1 = "";
        $FastFaves::weapon2 = "";
        $FastFaves::weapon3 = "";
        $FastFaves::weapon4 = "";
        $FastFaves::weapon5 = "";
        $FastFaves::weapon6 = "";
        $FastFaves::extra1 = "";
        $FastFaves::extra2 = "";
        $FastFaves::extra3 = "";
        $FastFaves::extra4 = "";
        $FastFaves::extra5 = "";
        $FastFaves::extra6 = "";
        $FastFaves::extra7 = "";
        $FastFaves::grenades = false;
        $FastFaves::mines = false;
        $FastFaves::tlaser = false;
        $FastFaves::beacons = false;

        if($Station::AtStation)
            FastFaves::buyFavorites();
        else
            FastFaves::PreloadMessage(5);
    }


    // This is a little weird; if our buy request is made before a previous request has finished,
    // the new request will be lost.  So, let's assume this is the case, and prepare to make the
    // same request again in 0.3 seconds.  This is a magic number I have determined as the
    // shortest interval that will work on fast servers.  (Slow servers are taken care of
    // by the Ping::Wait call that follows.)

    if($Station::AtStation)
    {
        schedule::cancel("FastFaves::redo();");
        schedule::add("FastFaves::redo();", 0.3);
    }
}


// Try to buy the favorites again if the first attempt was "lost"
function FastFaves::redo()
{
    // Only try again if we're still waiting for the request to finish,
    // and remember the ping ID so we can cancel the Ping::Wait function call
    // if the request beats the lag delay.

    if($FastFaves::Handling)
        $FastFaves::PingID = Ping::Wait("FastFaves::do($FastFaves::doing::device, $FastFaves::doing::m1, $FastFaves::doing::m2, $FastFaves::doing::m3, $FastFaves::doing::key);");

    return;
}


function FastFaves::onBuySuccessful()
{
    if($FastFaves::PingID) // we may still have a Ping::Wait pending
    {
        // Cancel the Ping::Wait, if it isn't too late
        Ping::cancel($FastFaves::PingID);
        $FastFaves::PingID = 0;
    }

    // Cancel the schedule if we're REALLY early :)
    schedule::cancel("FastFaves::redo();");

    // It's now safe to try another request.  Honest. :)
    $FastFaves::Handling = 0;

    return;
}


function FastFaves::validate()
{
    if(String::ICompare($FastFaves::pack, "Energy Pack")) // !=
    {
        %found = 0;
        // remove Laser Rifle from our loadout (or it will buy back the Energy Pack)

        for(%i = 1; %i < 7; %i++)
        {
            if(!String::ICompare($FastFaves::weapon[%i], "Laser Rifle")) // ==
            {
                while($FastFaves::weapon[%i] != "")
                {
                    $FastFaves::weapon[%i] = $FastFaves::weapon[%i + 1];
                    %i++;
                }

                %found++;
                break;
            }
        }

        if(%found)
        {
            $FastFaves::loadname = "";

            // rebuild $FastFaves::weaponstring
            $FastFaves::weaponString = "";
            for(%i = 1; %i <= 6; %i++)
            {
                if($FastFaves::weapon[%i] == "")
                    break;

                $FastFaves::weaponString = $FastFaves::weaponString @ $FastFaves::weapon[%i];
            }
        }
    }
}


// This loops whenever (and as long as) we're at an inventory station
function FastFaves::ScheduleLoop()
{
    if($Station::AtStation)
    {
        if(!$FastFaves::Handling || $FastFaves::Handling > 10)
        {
            if(Queue::count(FastFaves_key)) // we have queued purchase requests pending
            {
                %device = Queue::pop(FastFaves_device);
                %m1     = Queue::pop(FastFaves_m1);
                %m2     = Queue::pop(FastFaves_m2);
                %m3     = Queue::pop(FastFaves_m3);
                %key    = Queue::pop(FastFaves_key);
                FastFaves::do(%device, %m1, %m2, %m3, %key);
            }
        }
        else
        {
            $FastFaves::Handling++;
        }

        // Update the FastFavesHUD !!
        FastFaves::VerifyLoadout();

        // This flag gets cleared when we exit the inventory station
        if($FastFaves::Loop)
            schedule::add("FastFaves::ScheduleLoop();", 0.2);
    }
}


function FastFaves::verifyLoadout()
{
    // Clear the loadout name if the armor, pack, weapons, or extras or don't match the selected loadout

    $FastFaves::temp_name = $FastFaves::loadname;

    if(String::ICompare($FastFaves::armor, loadout::armor())) // !=
        $FastFaves::temp_name = "";

    if(String::ICompare($FastFaves::pack, loadout::pack())) // !=
        $FastFaves::temp_name = "";

    if(String::ICompare($FastFaves::weaponString, loadout::weapons())) // !=
        $FastFaves::temp_name = "";

    if(String::ICompare($FastFaves::extraString, loadout::extras())) // !=
        $FastFaves::temp_name = "";

    if($FastFaves::NoUnwantedAmmo)
    {
        if(!String::ICompare($loadout::pack, "Ammo Pack")) // ==
        {
            loadout::extraAmmo();
            for(%i = 1; %i <= $loadout::extraAmmoCount; %i++)
            {
                echo("$FastFaves::WantedAmmo[" @ $loadout::extraAmmo[%i] @ "] = " @ $FastFaves::WantedAmmo[$loadout::extraAmmo[%i]]);

                if(!$FastFaves::WantedAmmo[$loadout::extraAmmo[%i]])
                    sell($loadout::extraAmmo[%i]);
            }
        }
    }

    FastFaves::LoadoutMessage();
    return;
}


function FastFaves::onExitStation()
{
    FastFavesHUD::remove();
    FastFaves::AutoMount();

    $FastFaves::Loop = false;
    $FastFaves::Handling = 0;
    Queue::flush(FastFaves_device);
    Queue::flush(FastFaves_m1);
    Queue::flush(FastFaves_m2);
    Queue::flush(FastFaves_m3);
    Queue::flush(FastFaves_key);
    return;
}


function FastFaves::onCouldNotBuy(%itemtype, %itemname)
{
    // Prevent "APC ready" message when not at a vehicle station
    if(!String::ICompare(%itemname, $FastFaves::Vehicle)) // %itemname == $FastFaves::Vehicle
    {
        schedule::cancel(APC_ready_message);
        return mute;
    }

    return;
}


// Automatically buy the previously selected loadout if AutoBuy mode is on
// or automatically buy mines, grenades, beacons and a repair kit whenever you use an inventory station
function FastFaves::AutoBuy()
{
    Event::Trigger(eventBeforeFFAutoBuy);

    if($FastFaves::AutoBuyVehicle)
    {
        if(!String::ICompare($FastFaves::Vehicle, "Scout"))
        {
            buyType($Inv::Scout_Vehicle);
        }
        else
        {
            buy($FastFaves::Vehicle);
        }

        if($FastFaves::AnnounceAPC)
        {
            if(!String::ICompare($FastFaves::Vehicle, "LPC")) // ==
            {
                if(Flood::protect(LPC_ready_message, 10))
                {
                    if(String::ICompare($loadout::armor, "Light Armor")) // !=
                        schedule::add("say(1, \"LPC ready to go, waiting for 1 passenger + 1 pilot.~wwaitpas\");", 3, APC_ready_message);
                    else
                        schedule::add("say(1, \"LPC ready to go, waiting for 2 passengers.~wwaitpas\");", 3, APC_ready_message);
                }
            }
            else if(!String::ICompare($FastFaves::Vehicle, "HPC")) // ==
            {
                if(Flood::protect(HPC_ready_message, 10))
                {
                    if(String::ICompare($loadout::armor, "Light Armor")) // !=
                        schedule::add("say(1, \"HPC ready to go, waiting for 3 passengers + 1 pilot.~wwaitpas\");", 3, APC_ready_message);
                    else
                        schedule::add("say(1, \"HPC ready to go, waiting for 4 passengers.~wwaitpas\");", 3, APC_ready_message);
                }
            }
        }
    }

    if($FastFaves::AutoBuy)
        FastFaves::buyFavorites();

    // Automatically close the inv menu as soon as it's displayed
    if($FastFaves::closeStation)
        $Mode::ExitInventory = true;
}


// Toggle automatic loadout buying mode on/off
function FastFaves::AutoBuyToggle()
{
    remoteBP(2048, "<L5>Auto Buy Favorites <f2>" @ wif($FastFaves::AutoBuy = !$FastFaves::AutoBuy, "ON", "OFF"),4);
}


// Automatically mount the desired weapon on exit
function FastFaves::AutoMount()
{
    if(getItemCount($FastFaves::mount) && ($FastFaves::mount != ""))
        use($FastFaves::mount); // Switch to the weapon
    else
        nextWeapon(); // Switch to the next available weapon

    return;
}


function FastFaves::buyFavorites()
{
    if($Station::AtStation)
        $FastFaves::Handling++;

    Event::Trigger(eventBuyFavorites);

    remoteEval(2048, buyFavorites, Inv::getItemType($FastFaves::Armor),
                                   Inv::getItemType($FastFaves::Pack),
                                   Inv::getItemType($FastFaves::Weapon1),
                                   Inv::getItemType($FastFaves::Weapon2),
                                   Inv::getItemType($FastFaves::Weapon3),
                                   Inv::getItemType($FastFaves::Weapon4),
                                   Inv::getItemType($FastFaves::Weapon5),
                                   Inv::getItemType($FastFaves::Weapon6),
                                   Inv::getItemType($FastFaves::Extra1),
                                   Inv::getItemType($FastFaves::Extra2),
                                   Inv::getItemType($FastFaves::Extra3),
                                   Inv::getItemType($FastFaves::Extra4),
                                   Inv::getItemType($FastFaves::Extra5),
                                   Inv::getItemType($FastFaves::Extra6),
                                   Inv::getItemType($FastFaves::Extra7),
                                   $Inv::Repair_Kit,
                                   wif($FastFaves::tlaser, $Inv::Targeting_Laser, ""),
                                   wif($FastFaves::mines, $Inv::Mine, ""),
                                   wif($FastFaves::grenades, $Inv::Grenade, ""),
                                   wif($FastFaves::beacons, $Inv::Beacon, ""));

    // Make sure we get the max number of mines and grenades with an ammo pack
    // (buyFavorites won't get the max number of mines and grenades possible
    // if we're ordering an ammo pack, but buyItem will)
    if(!String::ICompare($FastFaves::Pack, "Ammo Pack")) // $FastFaves::Pack == "Ammo Pack"
    {
        schedule::add("buy(Mine);", 0.1);
        schedule::add("buy(Grenade);", 0.2);
    }
    return;
}


// Remove the loadout message when we leave the PlayGui,
// and display it again when we return
function FastFaves::onGuiOpen(%gui)
{
    if(%gui != PlayGui)
    {
        FastFavesHUD::remove();
    }
    else
    {
        if($Station::AtStation)
            FastFaves::LoadoutMessage();
    }

    return;
}


function FastFaves::LoadoutMessage()
{
    if(!$Station::AtStation)
        return;

    if(!$Mode::PlayMode)
        return;

    %grenades = getItemCount("Grenade");
    %mines    = getItemCount("Mine");
    %beacons  = getItemCount("Beacon");
    %tlaser   = getItemCount("Targeting Laser");

    if(%grenades)
        $FastFavesGRENADES::text = "<jc><f0>Grenades: <f1>" @ %grenades;
    else
        $FastFavesGRENADES::text = "<jl><f0>Grenades: <b0,5:writer\\b\\x_red.bmp>";

    if(%mines)
        $FastFavesMINES::text = "<jc><f0>Mines: <f1>" @ %mines;
    else
        $FastFavesMINES::text = "<jc><f0>Mines:  <b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\1x1.bmp><b0,5:writer\\b\\x_red.bmp>";

    if(%beacons)
        $FastFavesBEACONS::text = "<jc><f0>Beacons: <f1>" @ %beacons;
    else
        $FastFavesBEACONS::text = "<jr><f0>Beacons:  <b0,5:writer\\b\\x_red.bmp>";

    if(%tlaser)
        %tlaser_str = "<v0>Targeting Laser: <b0,5:writer\\b\\check_green.bmp>";
    else
        %tlaser_str = "<v0>Targeting Laser: <b0,5:writer\\b\\x_red.bmp>";


    $FastFavesMAIN::text = "<jc><f2>" @ $FastFaves::temp_name @ "<f0>\n<jl>"
                         @ "<f0>Armor:\t\t\t<f1>" @ $loadout::armor                                @ "<f0>\n"
                         @ "<f0>Pack:\t\t\t<f1>"  @ $loadout::pack                                 @ "<f0>\n"
                         @ "<f0>AutoDrop:\t<f1>"  @ $FastFaves::autodrop                           @ "<f0>\n"
                         @ "<f0>Litterbug:\t\t"   @ wif($litterbug::next::enabled, "<f2>", "<f1>")
                                                  @ $FastFaves::litterbug                          @ "<f0>\n"
                         @ "<f0>Vehicle:\t\t<f1>" @ $FastFaves::Vehicle                            @ "<f0>\n";

    $FastFavesEXTRAS::text = "<jl><f1>" @ $loadout::extra1 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra2 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra3 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra4 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra5 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra6 @ "<f0>\n"
                           @     "<f1>" @ $loadout::extra7;

    $FastFavesWEAPONS::text = "<jl>" @ wif(String::ICompare($FastFaves::mount, $loadout::weapon1), "<f1>", "<f2>") @ $loadout::weapon1 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $loadout::weapon2), "<f1>", "<f2>") @ $loadout::weapon2 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $loadout::weapon3), "<f1>", "<f2>") @ $loadout::weapon3 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $loadout::weapon4), "<f1>", "<f2>") @ $loadout::weapon4 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $loadout::weapon5), "<f1>", "<f2>") @ $loadout::weapon5 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $loadout::weapon6), "<f1>", "<f2>") @ $loadout::weapon6 @ "<f0>\n"
                                     @ %tlaser_str;

    FastFavesHUD::make();
    return;
}


function FastFaves::PreloadMessage(%delay)
{
    if($FastFaves::grenades)
        $FastFavesGRENADES::text = "<jl><f0>Grenades: <b0,5:writer\\b\\check_green.bmp>";
    else
        $FastFavesGRENADES::text = "<jl><f0>Grenades: <b0,5:writer\\b\\x_red.bmp>";

    if($FastFaves::mines)
    {
        $FastFavesMINES::text = "<jc><f0>Mines:  <b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\1x1.bmp><b0,5:writer\\b\\check_green.bmp>";
    }
    else
    {
        $FastFavesMINES::text = "<jc><f0>Mines:  <b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\10x10.bmp><b0,0:writer\\blanks\\1x1.bmp><b0,5:writer\\b\\x_red.bmp>";
    }

    if($FastFaves::beacons)
        $FastFavesBEACONS::text = "<jr><f0>Beacons:  <b0,5:writer\\b\\check_green.bmp>";
    else
        $FastFavesBEACONS::text = "<jr><f0>Beacons:  <b0,5:writer\\b\\x_red.bmp>";

    if($FastFaves::tlaser)
        %tlaser_str = "<v0>Targeting Laser: <b0,5:writer\\b\\check_green.bmp>";
    else
        %tlaser_str = "<v0>Targeting Laser: <b0,5:writer\\b\\x_red.bmp>";

    $FastFavesMAIN::text = "<jc><f2>" @ $FastFaves::loadname @ "<f0>\n<jl>"
                         @ "<f0>Armor:\t\t\t<f1>" @ $FastFaves::armor                              @ "<f0>\n"
                         @ "<f0>Pack:\t\t\t<f1>"  @ $FastFaves::pack                               @ "<f0>\n"
                         @ "<f0>AutoDrop:\t<f1>"  @ $FastFaves::autodrop                           @ "<f0>\n"
                         @ "<f0>Litterbug:\t\t"   @ wif($litterbug::next::enabled, "<f2>", "<f1>")
                                                  @ $FastFaves::litterbug                          @ "<f0>\n"
                         @ "<f0>Vehicle:\t\t<f1>" @ $FastFaves::Vehicle                            @ "<f0>\n";

    $FastFavesEXTRAS::text = "<jl><f1>" @ $FastFaves::extra1 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra2 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra3 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra4 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra5 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra6 @ "<f0>\n"
                           @     "<f1>" @ $FastFaves::extra7;

    $FastFavesWEAPONS::text = "<jl>" @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon1), "<f1>", "<f2>") @ $FastFaves::weapon1 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon2), "<f1>", "<f2>") @ $FastFaves::weapon2 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon3), "<f1>", "<f2>") @ $FastFaves::weapon3 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon4), "<f1>", "<f2>") @ $FastFaves::weapon4 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon5), "<f1>", "<f2>") @ $FastFaves::weapon5 @ "<f0>\n"
                                     @ wif(String::ICompare($FastFaves::mount, $FastFaves::weapon6), "<f1>", "<f2>") @ $FastFaves::weapon6 @ "<f0>\n"
                                     @ %tlaser_str;

    FastFavesHUD::make();

    FastFaves::buyFavorites();

    if(%delay)
        schedule::add("FastFavesHUD::remove();", %delay);

    return;
}


function FastFavesHUD::setup()
{
    $FastFavesGRENADES::x = 2;
    $FastFavesBEACONS::x = 2 * (whud::width($FastFavesMAIN::width)/3);

    $FastFavesMINES::x = whud::width($FastFavesMAIN::width)/3 - 2;
    $FastFavesMINES::width = whud::width($FastFavesMAIN::width)/3 + 2;

    $FastFavesGRENADES::width = whud::width($FastFavesMAIN::width)/3;
    $FastFavesBEACONS::width = whud::width($FastFavesMAIN::width)/3 - 4;

    return;
}
FastFavesHUD::setup();


function FastFavesHUD::make()
{
    wHUD::makeMoveable(true, true, false, FastFavesMAIN, $FastFavesMAIN::x, $FastFavesMAIN::y, $FastFavesMAIN::width, $FastFavesMAIN::height, false, true, $FastFavesMAIN::text, 2, 0, 2);//, %padLeft, %padTop, %padRight, %padBottom, %gui)
    wHUD::makeMoveable(true, true, false, FastFavesEXTRAS, $FastFavesEXTRAS::x, $FastFavesEXTRAS::y, $FastFavesEXTRAS::width, $FastFavesEXTRAS::height, false, true, $FastFavesEXTRAS::text, 2, 0, 2);//, %padLeft, %padTop, %padRight, %padBottom, %gui)
    wHUD::makeMoveable(true, true, false, FastFavesWEAPONS, $FastFavesWEAPONS::x, $FastFavesWEAPONS::y, $FastFavesWEAPONS::width, $FastFavesWEAPONS::height, false, true, $FastFavesWEAPONS::text, 2, 0, 2);//, %padLeft, %padTop, %padRight, %padBottom, %gui)

    %y = 6 * 14 - 4;
    %height = 14;

    $FastFavesGRENADES_CELL = newObject(FastFavesGRENADES_CELL, SimGui::Control, $FastFavesGRENADES::x + 4, -2 + %y, $FastFavesGRENADES::width, %height + 2);
    $FastFavesGRENADES_TEXT = newObject(FastFavesGRENADES_TEXT, FearGuiFormattedText, 0, 0, %width, %height);

    addToSet($wHUD::Frame[FastFavesMAIN], $FastFavesGRENADES_CELL);
    addToSet($FastFavesGRENADES_CELL, $FastFavesGRENADES_TEXT);

    $FastFavesMINES_CELL = newObject(FastFavesMINES_CELL, SimGui::Control, $FastFavesMINES::x, -2 + %y, $FastFavesMINES::width, %height + 2);
    $FastFavesMINES_TEXT = newObject(FastFavesMINES_TEXT, FearGuiFormattedText, 0, 0, %width, %height);

    addToSet($wHUD::Frame[FastFavesMAIN], $FastFavesMINES_CELL);
    addToSet($FastFavesMINES_CELL, $FastFavesMINES_TEXT);

    $FastFavesBEACONS_CELL = newObject(FastFavesBEACONS_CELL, SimGui::Control, $FastFavesBEACONS::x, -2 + %y, $FastFavesBEACONS::width, %height + 2);
    $FastFavesBEACONS_TEXT = newObject(FastFavesBEACONS_TEXT, FearGuiFormattedText, 0, 0, %width, %height);

    addToSet($wHUD::Frame[FastFavesMAIN], $FastFavesBEACONS_CELL);
    addToSet($FastFavesBEACONS_CELL, $FastFavesBEACONS_TEXT);


    Control::setValue(FastFavesGRENADES_TEXT, "<jc>" @ $FastFavesGRENADES::text);
    Control::setValue(FastFavesMINES_TEXT, "<jc>" @ $FastFavesMINES::text);
    Control::setValue(FastFavesBEACONS_TEXT, "<jc>" @ $FastFavesBEACONS::text);

    return;
}

function FastFavesHUD::remove()
{
    wHUD::remove(FastFavesEXTRAS);
    wHUD::remove(FastFavesWEAPONS);
    wHUD::remove(FastFavesMAIN);
    return;
}


function onHudMove_FastFavesMAIN(%x, %y, %width, %height)
{
    $FastFavesMAIN::x = %x;
    $FastFavesMAIN::y = %y;
    $FastFavesMAIN::width = %width;
    FastFavesHUD::setup();
    FastFavesHUD::make();
    return;
}

function onHudMove_FastFavesWEAPONS(%x, %y, %width, %height)
{
    $FastFavesWEAPONS::x = %x;
    $FastFavesWEAPONS::y = %y;
    $FastFavesWEAPONS::width = %width;
    FastFavesHUD::make();
    return;
}

function onHudMove_FastFavesEXTRAS(%x, %y, %width, %height)
{
    $FastFavesEXTRAS::x = %x;
    $FastFavesEXTRAS::y = %y;
    $FastFavesEXTRAS::width = %width;
    FastFavesHUD::make();
    return;
}


function FastFaves::to_temp()
{
    $FastFaves::temp::loadname = $FastFaves::loadname;
    $FastFaves::temp::pack     = $FastFaves::pack;
    $FastFaves::temp::armor    = $FastFaves::armor;

    $FastFaves::temp::weapon1  = $FastFaves::weapon1;
    $FastFaves::temp::weapon2  = $FastFaves::weapon2;
    $FastFaves::temp::weapon3  = $FastFaves::weapon3;
    $FastFaves::temp::weapon4  = $FastFaves::weapon4;
    $FastFaves::temp::weapon5  = $FastFaves::weapon5;
    $FastFaves::temp::weapon6  = $FastFaves::weapon6;

    $FastFaves::temp::mount    = $FastFaves::mount;

    $FastFaves::temp::extra1   = $FastFaves::extra1;
    $FastFaves::temp::extra2   = $FastFaves::extra2;
    $FastFaves::temp::extra3   = $FastFaves::extra3;
    $FastFaves::temp::extra4   = $FastFaves::extra4;
    $FastFaves::temp::extra5   = $FastFaves::extra5;
    $FastFaves::temp::extra6   = $FastFaves::extra6;
    $FastFaves::temp::extra7   = $FastFaves::extra7;

    $FastFaves::temp::grenades = $FastFaves::grenades;
    $FastFaves::temp::mines    = $FastFaves::mines;
    $FastFaves::temp::tlaser   = $FastFaves::tlaser;
    $FastFaves::temp::beacons  = $FastFaves::beacons;

    $FastFaves::temp::autodrop = $FastFaves::autodrop;
    $FastFaves::temp::litterbug = $FastFaves::litterbug;
    $FastFaves::temp::litternext = $litterbug::next::enabled;
}

function FastFaves::from_temp()
{
    $FastFaves::loadname = $FastFaves::temp::loadname;
    $FastFaves::pack     = $FastFaves::temp::pack;
    $FastFaves::armor    = $FastFaves::temp::armor;

    $FastFaves::weapon1  = $FastFaves::temp::weapon1;
    $FastFaves::weapon2  = $FastFaves::temp::weapon2;
    $FastFaves::weapon3  = $FastFaves::temp::weapon3;
    $FastFaves::weapon4  = $FastFaves::temp::weapon4;
    $FastFaves::weapon5  = $FastFaves::temp::weapon5;
    $FastFaves::weapon6  = $FastFaves::temp::weapon6;

    $FastFaves::mount    = $FastFaves::temp::mount;

    $FastFaves::extra1   = $FastFaves::temp::extra1;
    $FastFaves::extra2   = $FastFaves::temp::extra2;
    $FastFaves::extra3   = $FastFaves::temp::extra3;
    $FastFaves::extra4   = $FastFaves::temp::extra4;
    $FastFaves::extra5   = $FastFaves::temp::extra5;
    $FastFaves::extra6   = $FastFaves::temp::extra6;
    $FastFaves::extra7   = $FastFaves::temp::extra7;

    $FastFaves::grenades = $FastFaves::temp::grenades;
    $FastFaves::mines    = $FastFaves::temp::mines;
    $FastFaves::tlaser   = $FastFaves::temp::tlaser;
    $FastFaves::beacons  = $FastFaves::temp::beacons;

    $FastFaves::autodrop = $FastFaves::temp::autodrop;
    $FastFaves::litterbug = $FastFaves::temp::litterbug;
    $litterbug::next::enabled = $FastFaves::temp::litternext;
}

function FastFaves::clear(%all)
{
    $FastFaves::loadname = "";
    $FastFaves::pack     = "";
    $FastFaves::armor    = "";

    $FastFaves::weapon1  = "";
    $FastFaves::weapon2  = "";
    $FastFaves::weapon3  = "";
    $FastFaves::weapon4  = "";
    $FastFaves::weapon5  = "";
    $FastFaves::weapon6  = "";
    $FastFaves::mount    = "";

    $FastFaves::extra1   = "";
    $FastFaves::extra2   = "";
    $FastFaves::extra3   = "";
    $FastFaves::extra4   = "";
    $FastFaves::extra5   = "";
    $FastFaves::extra6   = "";
    $FastFaves::extra7   = "";

    $FastFaves::grenades = "";
    $FastFaves::mines    = "";
    $FastFaves::tlaser   = "";
    $FastFaves::beacons  = "";
    $FastFaves::vehicle  = "";

    if(%all)
    {
        $FastFaves::autodrop = "";
        $FastFaves::litterbug = "";
        $litterbug::next::enabled = false;
    }
}


function FastFaves::StoreLoadout(%device, %m1, %m2, %m3, %key)
{
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, name]       = $FastFaves::loadname;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, armor]      = $FastFaves::armor;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, pack]       = $FastFaves::pack;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon1]    = $FastFaves::weapon1;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon2]    = $FastFaves::weapon2;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon3]    = $FastFaves::weapon3;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon4]    = $FastFaves::weapon4;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon5]    = $FastFaves::weapon5;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, weapon6]    = $FastFaves::weapon6;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, mount]      = $FastFaves::mount;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, grenades]   = $FastFaves::grenades;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, mines]      = $FastFaves::mines;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, beacons]    = $FastFaves::beacons;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, tlaser]     = $FastFaves::tlaser;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra1]     = $FastFaves::extra1;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra2]     = $FastFaves::extra2;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra3]     = $FastFaves::extra3;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra4]     = $FastFaves::extra4;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra5]     = $FastFaves::extra5;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra6]     = $FastFaves::extra6;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, extra7]     = $FastFaves::extra7;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, autodrop]   = $FastFaves::autodrop;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, litterbug]  = $FastFaves::litterbug;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, litternext] = $litterbug::next::enabled;
    $FastFaves::loadout[%device, %m1, %m2, %m3, %key, vehicle]    = $FastFaves::vehicle;
    return;
}


function FastFaves::configMode()
{
    // remember current loadout?

    FastFaves::mainMenu();
}


function FastFaves::mainMenu()
{
    menu::reset();
    menu::additem("Save loadouts to favorites" @ $FastFaves::CurrentFileNumber @ ".cs");
    menu::additem("Create loadout based on current \"active\" loadout");
    menu::additem("Create loadout based on carried items");
    menu::additem("Create a \"single item\" loadout");
    menu::additem("Select loadout to use as default");

    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", FastFaves::mainmenu::onSelected, FastFaves::mainmenu::onCancelled, "", $FastFaves::mainmenu::selected, false, "", false, "", "50%", "a:FastFavesMAIN");
}


function FastFaves::mainmenu::onSelected(%index)
{
    FastFavesHUD::remove(); // remove the FastFavesHUD in case we've returned to this menu from a sub-menu

    $FastFaves::mainmenu::selected = %index;

    if(%index == 0) // Save loadouts
    {
        FastFaves::saveData();
        remoteBP(2048, "<jc><f2>favorites" @ $FastFaves::CurrentFileNumber @ ".cs saved", 2);
        schedule("FastFaves::mainMenu();", 2); // return to menu
    }
    else if(%index == 1) // Create loadout based on current "active" loadout
    {
        FastFaves::loadoutMenu();
    }
    else if(%index == 2) // Create loadout based on carried items
    {
        FastFaves::to_temp();

        $FastFaves::loadname = "";
        $FastFaves::armor = loadout::armor();
        $FastFaves::pack = loadout::pack();

        loadout::weapons();
        $FastFaves::weapon1 = $loadout::weapon1;
        $FastFaves::weapon2 = $loadout::weapon2;
        $FastFaves::weapon3 = $loadout::weapon3;
        $FastFaves::weapon4 = $loadout::weapon4;
        $FastFaves::weapon5 = $loadout::weapon5;
        $FastFaves::weapon6 = $loadout::weapon6;

        $FastFaves::mount = $Inv::Name[getMountedItem(0)];

        loadout::extras();
        $FastFaves::extra1 = $loadout::extra1;
        $FastFaves::extra2 = $loadout::extra2;
        $FastFaves::extra3 = $loadout::extra3;
        $FastFaves::extra4 = $loadout::extra4;
        $FastFaves::extra5 = $loadout::extra5;
        $FastFaves::extra6 = $loadout::extra6;
        $FastFaves::extra7 = $loadout::extra7;

        if(getItemCount("Grenade"))
            $FastFaves::grenades = true;
        else
            $FastFaves::grenades = false;

        if(getItemCount("Mine"))
            $FastFaves::mines = true;
        else
            $FastFaves::mines = false;

        if(getItemCount("Targeting Laser"))
            $FastFaves::tlaser = true;
        else
            $FastFaves::tlaser = false;

        if(getItemCount("Beacon"))
            $FastFaves::beacons = true;
        else
            $FastFaves::beacons = false;

        $FastFaves::loadoutMenu::from_temp_on_cancelled = true;

        FastFaves::loadoutMenu();
    }
    else if(%index == 3) // Create a "single item" loadout
    {
        FastFaves::to_temp();
        $FastFaves::singleMenu::from_temp_on_cancelled = true;
        FastFaves::clear(true);
        FastFaves::singleMenu();
    }
    else if(%index == 4) // Select loadout to use as default
    {
        bind::getkey("Select the key combination of the loadout to use\nwhen you first connect to a server", FastFaves::default::keySelected, FastFaves::mainMenu, false);
    }
    return;
}

function FastFaves::default::keySelected(%device, %m1, %m2, %m3, %key)
{
    $FastFaves::default::device = %device;
    $FastFaves::default::m1 = %m1;
    $FastFaves::default::m2 = %m2;
    $FastFaves::default::m3 = %m3;
    $FastFaves::default::key = %key;
    FastFaves::mainMenu();
    return;
}


function FastFaves::mainmenu::onCancelled()
{
    FastFavesHUD::remove();
    return;
}


function FastFaves::getItemgroup(%onSelected, %onCancelled, %itemgroup)
{
    FastFavesHUD::remove();
    menu::reset();
    menu::copyArray("$Litterbug::itemgroup[%1, name]", 0, $Litterbug::itemgroups - 1);

    menu::additem("( NONE )");

//    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", %onSelected, %onCancelled, $Litterbug::itemsPerPage, %itemgroup);
    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", %onSelected, %onCancelled, "", %itemgroup);
}


function FastFaves::getVehicle(%onSelected, %onCancelled)
{
    FastFavesHUD::remove();
    menu::reset();

    menu::copyArray("$Inv::vehicleName::list", 1, $Inv::vehicleName::count);
    menu::additem("( NONE )");

    %vehicle_index = $Inv::vehicleName::count;

    // Preselect the current FastFaves vehicle
    if(String::Compare($FastFaves::Vehicle, "")) // !=
    {
        for(%i = 0; %i < $Inv::vehicleName::count; %i++)
        {
            if(!String::ICompare($Inv::vehicleName::list[%i + 1], $FastFaves::Vehicle)) // ==
            {
                %vehicle_index = %i;
                break;
            }
        }
    }
    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", %onSelected, %onCancelled, "", %vehicle_index);
}


function FastFaves::loadoutMenu()
{
    FastFaves::PreloadMessage();

    menu::reset();

    menu::additem("Assign this loadout to a key (perform key bind)");

    if(String::Compare($FastFaves::loadname, "")) // !=
        menu::additem("Rename loadout");
    else
        menu::additem("Name loadout");

    menu::additem("Select autodrop itemgroup");
    menu::additem("Select litterbug itemgroup");

    if($litterbug::next::enabled)
        menu::additem("Disable one-time litterbuggin' at next station access");
    else
        menu::additem("Enable one-time litterbuggin' at next station access");

    menu::additem("Use whatever one-time litterbuggin' setting is in effect at the time");

    menu::additem("Select vehicle");

    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", FastFaves::loadoutMenu::onSelected, FastFaves::loadoutMenu::onCancelled, "", $FastFaves::loadoutMenu::selected, false, "", false, "", "50%", "a:FastFavesMAIN");
}


function FastFaves::loadoutMenu::onSelected(%index)
{
    $FastFaves::loadoutMenu::selected = %index;
    $FastFaves::loadoutMenu::from_temp_on_cancelled = false;

    if(%index == 0) // Assign loadout to a key
    {
        bind::getkey("Select a key for this loadout", FastFaves::loadoutMenu::keySelected, FastFaves::loadoutMenu, true);
    }
    else if(%index == 1) // Name/Rename loadout
    {
        ask("Enter a name for this loadout:", "<jc><f2>", $FastFaves::loadname, FastFaves::loadoutMenu::asked_name, FastFaves::loadoutMenu, "50%", "a:FastFavesMAIN");
    }
    else if(%index == 2) // Select autodrop itemgroup
    {
        FastFaves::getItemgroup(FastFaves::loadoutMenu::autodrop::onSelected, FastFaves::loadoutMenu, $Litterbug[$FastFaves::autodrop]);
    }
    else if(%index == 3) // Select litterbug itemgroup
    {
        FastFaves::getItemgroup(FastFaves::loadoutMenu::litterbug::onSelected, FastFaves::loadoutMenu, $Litterbug[$FastFaves::litterbug]);
    }
    else if(%index == 4) // Disable/enable litternext
    {
        $litterbug::next::enabled = !$litterbug::next::enabled;
        FastFaves::loadoutMenu();
    }
    else if(%index == 5) // Clear litternext
    {
        $litterbug::next::enabled = "";
        FastFaves::loadoutMenu();
    }
    else if(%index == 6) // Select vehicle
    {
        FastFaves::getVehicle(FastFaves::loadoutMenu::vehicle::onSelected, FastFaves::loadoutMenu);
    }
    return;
}


function FastFaves::loadoutMenu::keySelected(%device, %m1, %m2, %m3, %key)
{
    echo("device = " @ %device);
    echo("m1 = " @ %m1);
    echo("m2 = " @ %m2);
    echo("m3 = " @ %m3);
    echo("key = " @ %key);

    FastFaves::StoreLoadout(%device, %m1, %m2, %m3, %key);
    FastFaves::bind(%device, %m1, %m2, %m3, %key);

    FastFaves::loadoutMenu();
    return;
}


function FastFaves::loadoutMenu::onCancelled()
{
    if($FastFaves::loadoutMenu::from_temp_on_cancelled)
        FastFaves::from_temp();

    $FastFaves::loadoutMenu::from_temp_on_cancelled = false;
    FastFaves::mainMenu();
    return;
}


function FastFaves::loadoutMenu::autodrop::onSelected(%index)
{
    if(%index < $Litterbug::itemgroups)
        $FastFaves::autodrop = $Litterbug::itemgroup[%index, name];
    else
        $FastFaves::autodrop = "";

    FastFaves::loadoutMenu();
    return;
}


function FastFaves::loadoutMenu::litterbug::onSelected(%index)
{
    if(%index < $Litterbug::itemgroups)
        $FastFaves::litterbug = $Litterbug::itemgroup[%index, name];
    else
        $FastFaves::litterbug = "";

    FastFaves::loadoutMenu();
    return;
}


function FastFaves::loadoutMenu::asked_name(%value)
{
    $FastFaves::loadname = %value;
    FastFaves::loadoutMenu();
    return;
}


function FastFaves::loadoutMenu::vehicle::onSelected(%index)
{
    // translate 0-based index to 1-based index
    %index++;

    if(%index > $Inv::vehicleName::Count)
        $FastFaves::vehicle = "";
    else
        $FastFaves::vehicle = $Inv::vehicleName::List[%index];

    FastFaves::loadoutMenu();
    return;
}


function FastFaves::singleMenu()
{
    FastFaves::PreloadMessage();

    menu::reset();

    menu::additem("Assign this single-item loadout to a key (perform key bind)");

    if(String::Compare($FastFaves::loadname, "")) // !=
        menu::additem("Rename loadout");
    else
        menu::additem("Name loadout");

    menu::additem("Select an armor");
    menu::additem("Select a pack");
    menu::additem("Select a weapon");
    menu::additem("Select a tool");
    menu::additem("Select a vehicle");
    menu::additem("Select ammo");

    menu::additem("Select autodrop itemgroup");
    menu::additem("Select litterbug itemgroup");

    if($litterbug::next::enabled)
        menu::additem("Disable one-time litterbuggin' at next station access");
    else
        menu::additem("Enable one-time litterbuggin' at next station access");

    menu::additem("Use whatever one-time litterbuggin' setting is in effect at the time");

    menu::make("<jc><f1>Select an item or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::onSelected, FastFaves::singleMenu::onCancelled, "", $FastFaves::singleMenu::selected, false, "", false, "", "50%", "a:FastFavesMAIN");
}


function FastFaves::singleMenu::onSelected(%index)
{
    $FastFaves::singleMenu::selected = %index;
    $FastFaves::singleMenu::from_temp_on_cancelled = false;

    if(%index == 0) // Assign loadout to a key
    {
        bind::getkey("Select a key for this single-item loadout", FastFaves::singleMenu::keySelected, FastFaves::singleMenu, true);
    }
    else if(%index == 1) // Name/Rename loadout
    {
        ask("Enter a name for this loadout:", "<jc><f2>", $FastFaves::loadname, FastFaves::singleMenu::asked_name, FastFaves::singleMenu, "50%", "a:FastFavesMAIN");
    }
    else if(%index == 2) // Select an armor
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::armorName::List[%1]", 1, $Inv::armorName::Count);

        // Preselect the current FastFaves armor
        if(String::Compare($FastFaves::Armor, "")) // !=
        {
            for(%i = 0; %i < $Inv::armorName::count; %i++)
            {
                if(!String::ICompare($Inv::armorName::list[%i + 1], $FastFaves::Armor)) // ==
                {
                    %armor_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select an armor or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::armor::onSelected, FastFaves::singleMenu, "", %armor_index);
    }
    else if(%index == 3) // Select a pack
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::packName::List[%1]", 1, $Inv::packName::Count);

        // Preselect the current FastFaves pack
        if(String::Compare($FastFaves::pack, "")) // !=
        {
            for(%i = 0; %i < $Inv::packName::count; %i++)
            {
                if(!String::ICompare($Inv::packName::list[%i + 1], $FastFaves::pack)) // ==
                {
                    %pack_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select a pack or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::pack::onSelected, FastFaves::singleMenu, "", %pack_index);
    }
    else if(%index == 4) // Select a weapon
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::weaponName::List[%1]", 1, $Inv::weaponName::Count);

        // Preselect the current FastFaves weapon
        if(String::Compare($FastFaves::weapon, "")) // !=
        {
            for(%i = 0; %i < $Inv::weaponName::count; %i++)
            {
                if(!String::ICompare($Inv::weaponName::list[%i + 1], $FastFaves::weapon)) // ==
                {
                    %weapon_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select a weapon or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::weapon::onSelected, FastFaves::singleMenu, "", %weapon_index);
    }
    else if(%index == 5) // Select a tool
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::toolName::List[%1]", 1, $Inv::toolName::Count);

        // Preselect the current FastFaves tool
        if(String::Compare($FastFaves::tool, "")) // !=
        {
            for(%i = 0; %i < $Inv::toolName::count; %i++)
            {
                if(!String::ICompare($Inv::toolName::list[%i + 1], $FastFaves::tool)) // ==
                {
                    %tool_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select a tool or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::tool::onSelected, FastFaves::singleMenu, "", %tool_index);
    }
    else if(%index == 6) // Select a vehicle
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::vehicleName::List[%1]", 1, $Inv::vehicleName::Count);

        // Preselect the current FastFaves vehicle
        if(String::Compare($FastFaves::vehicle, "")) // !=
        {
            for(%i = 0; %i < $Inv::vehicleName::count; %i++)
            {
                if(!String::ICompare($Inv::vehicleName::list[%i + 1], $FastFaves::vehicle)) // ==
                {
                    %vehicle_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select a vehicle or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::vehicle::onSelected, FastFaves::singleMenu, "", %vehicle_index);
    }
    else if(%index == 7) // Select ammo
    {
        FastFavesHUD::remove();
        menu::reset();
        menu::copyArray("$Inv::ammoName::List[%1]", 1, $Inv::ammoName::Count);

        // Preselect the current FastFaves ammo
        if(String::Compare($FastFaves::ammo, "")) // !=
        {
            for(%i = 0; %i < $Inv::ammoName::count; %i++)
            {
                if(!String::ICompare($Inv::ammoName::list[%i + 1], $FastFaves::ammo)) // ==
                {
                    %ammo_index = %i;
                    break;
                }
            }
        }
        menu::make("<jc><f1>Select ammo or press <f2>Esc<f1> to exit\n", FastFaves::singleMenu::ammo::onSelected, FastFaves::singleMenu, "", %ammo_index);
    }
    else if(%index == 8) // Select autodrop itemgroup
    {
        FastFaves::getItemgroup(FastFaves::singleMenu::autodrop::onSelected, FastFaves::singleMenu, $Litterbug[$FastFaves::autodrop]);
    }
    else if(%index == 9) // Select litterbug itemgroup
    {
        FastFaves::getItemgroup(FastFaves::singleMenu::litterbug::onSelected, FastFaves::singleMenu, $Litterbug[$FastFaves::litterbug]);
    }
    else if(%index == 10) // Disable/enable litternext
    {
        $litterbug::next::enabled = !$litterbug::next::enabled;
        FastFaves::singleMenu();
    }
    else if(%index == 11) // Clear litternext
    {
        $litterbug::next::enabled = "";
        FastFaves::singleMenu();
    }
    return;
}


function FastFaves::singleMenu::keySelected(%device, %m1, %m2, %m3, %key)
{
    echo("device = " @ %device);
    echo("m1 = " @ %m1);
    echo("m2 = " @ %m2);
    echo("m3 = " @ %m3);
    echo("key = " @ %key);

    FastFaves::StoreLoadout(%device, %m1, %m2, %m3, %key);
    FastFaves::bind(%device, %m1, %m2, %m3, %key);

    FastFaves::singleMenu();
    return;
}


function FastFaves::singleMenu::onCancelled()
{
    if($FastFaves::singleMenu::from_temp_on_cancelled)
        FastFaves::from_temp();

    $FastFaves::singleMenu::from_temp_on_cancelled = false;
    FastFaves::mainMenu();
    return;
}


function FastFaves::singleMenu::autodrop::onSelected(%index)
{
    if(%index < $Litterbug::itemgroups)
        $FastFaves::autodrop = $Litterbug::itemgroup[%index, name];
    else
        $FastFaves::autodrop = "";

    FastFaves::singleMenu();
    return;
}


function FastFaves::singleMenu::litterbug::onSelected(%index)
{
    if(%index < $Litterbug::itemgroups)
        $FastFaves::litterbug = $Litterbug::itemgroup[%index, name];
    else
        $FastFaves::litterbug = "";

    FastFaves::singleMenu();
    return;
}


function FastFaves::singleMenu::asked_name(%value)
{
    $FastFaves::loadname = %value;
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::armor::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::armor = $Inv::armorName::List[%index];
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::pack::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::pack = $Inv::packName::List[%index];
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::weapon::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::weapon = $Inv::weaponName::List[%index];
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::tool::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::tool = $Inv::toolName::List[%index];
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::vehicle::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::vehicle = $Inv::vehicleName::List[%index];
    FastFaves::singleMenu();
    return;
}

function FastFaves::singleMenu::ammo::onSelected(%index)
{
    FastFaves::clear();

    // translate 0-based index to 1-based index
    %index++;

    $FastFaves::ammo = $Inv::ammoName::List[%index];
    FastFaves::singleMenu();
    return;
}

function fast_faves::save()
{
    AddExportText("    // Set this to true if you want to force shutting the cursor off on eventExitStation.");
    AddExportText("    // This might solve the \"exit station, get cursor on screen\" bug which should not");
    AddExportText("    // happen unless you've enabled station_types.cs or have some other script that is");
    AddExportText("    // messing with things.  (I have been unable to duplicate the bug with the new fast");
    AddExportText("    // faves, under any circumstances, with only my scripts installed.)");
    ExportSetting("    $FastFaves::ForceCursorOff", false);
    AddExportText("");
    AddExportText("    // Set this to true to automatically buy the currently selected loadout");
    AddExportText("    // as soon as you access an inventory station.");
    ExportSetting("    $FastFaves::AutoBuy", true);
    AddExportText("");
    AddExportText("    // Set this to true to automatically buy the currently selected vehicle");
    AddExportText("    // whenever you access a vehicle station.");
    ExportSetting("    $FastFaves::AutoBuyVehicle", true);
    AddExportText("");
    AddExportText("    // Set this to true to automatically announce that an APC is ready to go");
    AddExportText("    // whenever you buy a vehicle");
    ExportSetting("    $FastFaves::AnnounceAPC", true);
    AddExportText("");
    AddExportText("    // Set this to true to prevent the inventory menu from popping up automatically");
    AddExportText("    // when you access an inventory station.  Note that it will still pop up momentarily");
    AddExportText("    // at vehicle stations, and at inventory stations on some servers. In those cases,");
    AddExportText("    // fast_favorites automatically closes the screen for you.  Note also that you can");
    AddExportText("    // always access the inventory menu using the \"I\" key.");
    ExportSetting("    $pref::noEnterInvStation", false);
    AddExportText("");
    AddExportText("    // Set this to true to automatically close the inv menu as soon as it's displayed");
    AddExportText("    // at vehicle stations (also useful on servers that don't respect $pref::noEnterInvStation)");
    ExportSetting("    $FastFaves::closeStation", false);
    AddExportText("");
    AddExportText("    // Set favorites file to use when you first load Tribes");
    AddExportText("    // Setting this to 0 will load config\\favorites0.cs");
    AddExportText("    // Setting this to 1 will load config\\favorites1.cs, etc.");
    ExportSetting("    $FastFaves::FileNumber", 0);
    AddExportText("");
    AddExportText("    // Set this to true to not reset your current loadout to the file's default");
    AddExportText("    // when you change to a different favorites?.cs file");
    ExportSetting("    $FastFaves::RememberCurrentLoadoutBetweenFavoritesFiles", false);
    AddExportText("");
    AddExportText("    // Note: the actual favorites key combos are defined in the favorites?.cs files");
    AddExportText("");
    AddExportText("    // Toggle the inventory screen on/off at any time");

    ExportBind("$Mode::ExitInventory=false; remoteEval(2048, ToggleInventoryMode);", "bind(keyboard0, make, \"i\", TO, \"$Mode::ExitInventory=false; remoteEval(2048, ToggleInventoryMode);\");");

    AddExportText("");
    AddExportText("    // Toggle auto-buying mode on/off (automatically buy selected loadout)");

    ExportBind("FastFaves::AutoBuyToggle();", "//bind(keyboard0, make, alt, \"numpad*\", TO, \"FastFaves::AutoBuyToggle();\");");
    ExportBind("FastFaves::AutoBuyToggle_break", "//bind(keyboard0, break, alt, \"numpad*\", TO, \"\", FastFaves::AutoBuyToggle_break);");

    AddExportText("");
    AddExportText("    // Press this key to select a favorites?.cs file (using a menu, no less :)");

    ExportBind("FastFaves::selectFavoritesFile();", "//bind(keyboard0, make, alt, \"numpad+\", TO, \"FastFaves::selectFavoritesFile();\");");
    ExportBind("FastFaves::selectFavoritesFile_break", "//bind(keyboard0, break, alt, \"numpad+\", TO, \"\", FastFaves::selectFavoritesFile_break);");

    AddExportText("");
    AddExportText("    // Enter FastFaves config mode (edit/create loadouts from in game)");

    ExportBind("FastFaves::ConfigMode();",    "//bind(keyboard0, make, control, \"numpadenter\", TO, \"FastFaves::ConfigMode();\");");
    ExportBind("FastFaves::ConfigMode_break", "//bind(keyboard0, break, control, \"numpadenter\", TO, \"\", FastFaves::ConfigMode_break);");

    AddExportText("");
    AddExportText("    // FastFaves HUD settings");
    AddExportText("    // You probably won't want to change these, but feel free if you know what you're doing :)");
    AddExportText("");
    ExportSetting("    $FastFavesMAIN::x", "50%");
    ExportSetting("    $FastFavesMAIN::y", "80%");
    ExportSetting("    $FastFavesMAIN::width", "50%");
    AddExportText("");
    ExportSetting("    $FastFavesEXTRAS::x", "l:FastFavesMAIN", "", "left of FastFavesMAIN");
    ExportSetting("    $FastFavesEXTRAS::y", "s:FastFavesMAIN", "", "same as FastFavesMAIN");
    ExportSetting("    $FastFavesEXTRAS::width", "20%");
    AddExportText("");
    ExportSetting("    $FastFavesWEAPONS::x", "r:FastFavesMAIN", "", "right of FastFavesMAIN");
    ExportSetting("    $FastFavesWEAPONS::y", "s:FastFavesMAIN", "", "same as FastFavesMAIN");
    ExportSetting("    $FastFavesWEAPONS::width", "20%");
}

WriterPrefs::save("fast_faves");

function fast_faves::init()
{
    needs("writer\\fast_faves.cs", "writer\\event.cs 2.7",
                                   "writer\\inventory.cs 3.3",
                                   "writer\\item_events.cs 2.12",
                                   "writer\\menu.cs 1.3",
                                   "writer\\modifier.cs 2.5",
                                   "writer\\ping.cs 3.1",
                                   "writer\\queue.cs 2.2",
                                   "writer\\save.cs 2.3",
                                   "writer\\schedule.cs 1.3",
                                   "writer\\station_events.cs 4.8",
                                   "writer\\whatif.cs 1.1");

    // Use default favorites after connecting to a game -- great idea Cowboy!
    if(!$FastFaves::RememberCurrentLoadoutBetweenFavoritesFiles)
        Event::Attach(eventFavoritesInitialized, "schedule::add(\"FastFaves($FastFaves::default::device,$FastFaves::default::m1,$FastFaves::default::m2,$FastFaves::default::m3,$FastFaves::default::key);\", 4);");

    Event::Attach(eventInventoryTableReady, "FastFaves::setupFavorites();");
    Event::Attach(eventEnterStation, "$FastFaves::Loop=true;FastFaves::ScheduleLoop();FastFaves::AutoBuy();");
    Event::Attach(eventExitStation, "FastFaves::onExitStation();");
    Event::Attach(eventBuySuccessful, "FastFaves::onBuySuccessful();");

    Event::Attach(eventConnected, "$FastFaves::Loop=false;");
    Event::Attach(eventChangeMission, "$FastFaves::Loop=false;");

    Event::Attach(eventCouldNotBuy, FastFaves::onCouldNotBuy);
    Event::Attach(eventCouldNotBuy_Flag, "return mute;");
    Event::Attach(eventCouldNotBuy_False, "return mute;");
    Event::Attach(eventGuiOpen, FastFaves::onGuiOpen);

    if($FastFaves::ForceCursorOff) Event::Attach(eventExitStation, "cursorOff(MainWindow);schedule::add(\"cursorOff(MainWindow);\", 1);");
    if($FastFaves::ForceCursorOff) Event::Attach(eventEnterStation, "schedule::cancel(\"cursorOff(MainWindow);\");");

    Event::Attach(eventScreenModeChanged, "FastFavesHUD::setup();");
    Event::Attach(eventScreenSizeChanged, "FastFavesHUD::setup();");
    Event::Attach(wHudDone, "FastFavesHUD::remove();");
    Event::Attach(wHudMove_FastFavesMAIN, onHudMove_FastFavesMAIN);
    Event::Attach(wHudMove_FastFavesEXTRAS, onHudMove_FastFavesEXTRAS);
    Event::Attach(wHudMove_FastFavesWEAPONS, onHudMove_FastFavesWEAPONS);

    Event::Attach(wHudSave_FastFavesMAIN, "WriterPrefs::save(\"fast_faves\");");
    Event::Attach(wHudSave_FastFavesEXTRAS, "WriterPrefs::save(\"fast_faves\");");
    Event::Attach(wHudSave_FastFavesWEAPONS, "WriterPrefs::save(\"fast_faves\");");
}
Event::Attach(writerInit, "fast_faves::init();");

if(!$UsingWriterLoader) fast_faves::init();
